@@ -1,6 +1,3 @@
-
-
-
 """This subpackage provides various modules to gain information about grammars
 
 It features multiple modules:
@@ -10,7 +7,7 @@ It features multiple modules:
 #. :mod:`classes.classifers.form_classifiers`
     Featuring functions of the format ``has_<X>_form``
 
-:examples:
+:example:
     >>> from formgram.grammars.str_interface import parse
     >>> from formgram.grammars.classifiers.chomsky_classifiers import get_chomsky_type
     >>> from formgram.grammars.classifiers.form_classifiers import has_chomsky_normal_form
