@@ -1,4 +1,6 @@
+"""This module provides decorators for functions
 
+"""
 
 import copy
 from functools import wraps
