@@ -1,14 +1,11 @@
-"""This module provides functions which work on any iterable
-
-Currently the only function provided is the powerset function
-
+"""This module provides functions which work on :class:`Collections`
 """
 
-
-
 import itertools
 from typing import Collection, Iterable
 
+from formgram.grammars.types import Symbol
+
 
 def powerset(collection: Collection) -> Iterable:
     """This function returns an iterable powerset of the provided Collection
@@ -31,13 +28,13 @@ def powerset(collection: Collection) -> Iterable:
     )
 
 
-def find_new_unique_string(previous_symbols: Collection, string_base: str) -> str:
+def find_new_unique_string(previous_symbols: Collection[Symbol], string_base: Symbol) -> Symbol:
     """Create a new string not in `previous_symbols` by adding numerals
 
     :rtype: object
     :param previous_symbols:
     :param string_base:
-    :return:
+    :return: a new string not in the collection of previous symbols
     """
     if string_base not in previous_symbols:
         return string_base
