@@ -1,6 +1,6 @@
 """This package uses yacc to parse strings into grammar dicts
 
-:examples:
+:example:
     >>> grammar_string = ''' # comments look like this and are permitted in the string
     >>> <START> ::= <A> | 's' <START>
     >>> 's''s'<A> ::= 's'<A>'s'
@@ -26,5 +26,4 @@
 """
 
 
-
 from formgram.grammars.str_interface.backus_naur_parser import parse
