@@ -1,16 +1,4 @@
-
-
-
 """This package holds modules providing form transformations of grammars
 
-
-These are
-
-#. context_free
-    Provides
-#. montotone.py
-
-#. regular.py
-
-
+The subpackages are organized by chomsky type.
 """
\ No newline at end of file
