@@ -4,20 +4,19 @@ It is assumed in this module that grammars are as described in :ref:`the package
 
 It is not checked if the grammars used as input are actually of type 1.
 """
-
-
+from typing import Set
 
 from formgram.grammars.classifiers.form_classifiers import has_separated_terminals, has_chomsky_normal_form
 from formgram.grammars.helper_functions.decorators import deepcopy_arguments
 from formgram.grammars.helper_functions.production_grouping import group_productions_by_producible_symbols, \
     group_right_hand_sides_by_left_hand_sides
 from formgram.grammars.helper_functions.set_functions import powerset
-from formgram.grammars.utility.context_free.helper import collect_nullable_symbols
+from formgram.grammars.types import GrammarDict, Symbol
 from formgram.grammars.utility.unrestricted.helper import find_new_nonterminal
 
 
 @deepcopy_arguments
-def to_separated_terminals_form(grammar: dict) -> dict:
+def to_separated_terminals_form(grammar: GrammarDict) -> GrammarDict:
     """Transform context-free grammar by separating terminals from variables.
 
     If the grammar is already of the correct form a deepcopy is returned.
@@ -66,8 +65,44 @@ def to_separated_terminals_form(grammar: dict) -> dict:
     }
 
 
+def _collect_nullable_symbols(grammar: GrammarDict) -> Set[Symbol]:
+    """Return the subset of nonterminals which can produce the empty word
+
+     This function iterates over the collection of productions collecting all
+     nonterminals which can directly produce the empty word.
+     This collection of nullable nonterminals is then expanded each iteration by
+     all nonterminals which can directly produce any word of only nullable symbols.
+     The iterations continue until no new nonterminals are found.
+
+    :examples:
+        >>> grammar_string = '''
+        >>> <S> ::= <A> <A> <A> | <A> <B> <A> |
+        >>> <A> ::=
+        >>> <B> ::= 'b'
+        >>> '''
+        >>> from formgram.grammars.str_interface import parse
+        >>> grammar_dict = parse(grammar_string)
+        >>> _collect_nullable_symbols(grammar_dict)
+        {"S", "A"}
+
+    :param grammar: The grammar to collect nullable nonterminals of
+    :return: The set of nullable nonterminals
+    """
+    nullable_symbols: Set[str] = set()
+    while True:
+        newly_found_nullable_symbols = set()
+        for (non_terminal,), right_hand_side in grammar["productions"]:
+            if non_terminal not in nullable_symbols and nullable_symbols.issuperset(
+                right_hand_side
+            ):
+                newly_found_nullable_symbols.add(non_terminal)
+        nullable_symbols.update(newly_found_nullable_symbols)
+        if not newly_found_nullable_symbols:
+            return nullable_symbols
+
+
 @deepcopy_arguments
-def to_epsilon_free_form(grammar: dict) -> dict:
+def to_epsilon_free_form(grammar: GrammarDict) -> GrammarDict:
     """Remove all epsilon rules from the grammar by transformation
 
     An exception to this is if the left hand side is the starting symbol.
@@ -82,8 +117,8 @@ def to_epsilon_free_form(grammar: dict) -> dict:
         >>> <B> ::= 'b'
         >>> '''
         >>> from formgram.grammars.str_interface import parse
-        >>> grammar = parse(grammar_string)
-        >>> to_epsilon_free_form(grammar)
+        >>> grammar_dict = parse(grammar_string)
+        >>> to_epsilon_free_form(grammar_dict)
         {
             "terminals": {'b'},
             "nonterminals": {'S', 'A', 'B'}
@@ -107,7 +142,7 @@ def to_epsilon_free_form(grammar: dict) -> dict:
     """
 
     # collect all symbols which can derive epsilon in any amount of steps
-    nullable_symbols = collect_nullable_symbols(grammar)
+    nullable_symbols = _collect_nullable_symbols(grammar)
 
     # remove direct epsilon productions
     # then for each production containing nullable symbols remove every combination
@@ -145,7 +180,7 @@ def to_epsilon_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_bifurcated_form(grammar: dict) -> dict:
+def to_bifurcated_form(grammar: GrammarDict) -> GrammarDict:
     """Make sure the right hand sides of the grammar have at most two symbols
 
     This Function trims all productions with long right hand sides to length two.
@@ -153,9 +188,11 @@ def to_bifurcated_form(grammar: dict) -> dict:
     them together.
 
     :example:
+        >>> from formgram.grammars.str_interface import parse
+        >>> from formgram.grammars.utility.unrestricted.helper import to_backus_naur_form
         >>> grammar_string = '''<A> ::= <B> 'c' <D> 'e' <F>'''
-        >>> grammar = parse(grammar_string)
-        >>> new_grammar = to_bifurcated_form(grammar)
+        >>> grammar_dict = parse(grammar_string)
+        >>> new_grammar = to_bifurcated_form(grammar_dict)
         >>> to_backus_naur_form(new_grammar)
         '''<A>   ::= <B> <A_1>
         <A_1> ::= 'c' <A_2>
@@ -195,18 +232,20 @@ def to_bifurcated_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_unit_free_form(grammar: dict) -> dict:
+def to_unit_free_form(grammar: GrammarDict) -> GrammarDict:
     """Remove all unit rules from the production set of the grammar
 
     That is accomplished by adding all right hand sides of the target of a unit
     rule to the image of the source.
 
     :example:
+        >>> from formgram.grammars.str_interface import parse
+        >>> from formgram.grammars.utility.unrestricted.helper import to_backus_naur_form
         >>> grammar_string = '''
         >>> <A> ::= <B> <C> | <D>
         >>> <D> ::= <E> <F>'''
-        >>> grammar = parse(grammar_string)
-        >>> new_grammar = to_unit_free_form(grammar)
+        >>> grammar_dict = parse(grammar_string)
+        >>> new_grammar = to_unit_free_form(grammar_dict)
         >>> to_backus_naur_form(new_grammar)
         '''
         <A> ::= <B> <C> | <E> <F>
@@ -274,7 +313,7 @@ def to_unit_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_new_start_symbol_form(grammar: dict) -> dict:
+def to_new_start_symbol_form(grammar: GrammarDict) -> GrammarDict:
     """Replace the old starting symbol with a new one only occurring on left hand sides
 
     This function takes a grammar and replaces its starting symbol with a new one.
@@ -298,7 +337,7 @@ def to_new_start_symbol_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_leftbound_start_form(grammar: dict) -> dict:
+def to_leftbound_start_form(grammar: GrammarDict) -> GrammarDict:
     """Assure that the starting symbol exists only on left hand sides.
 
     If the grammars starting symbol is already left bound, a copy of the input
@@ -318,7 +357,7 @@ def to_leftbound_start_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_monotone_start_form(grammar: dict) -> dict:
+def to_monotone_start_form(grammar: GrammarDict) -> GrammarDict:
     """Assure that the starting symbol can not be used to break monotonicity directly
 
     If there is a starting symbol production which breaks monotonicity the grammar
@@ -343,7 +382,7 @@ def to_monotone_start_form(grammar: dict) -> dict:
     # check if starting-symbol is in right hand sides
     start_symbol = grammar["starting_symbol"]
 
-    nullable_symbols = collect_nullable_symbols(grammar)
+    nullable_symbols = _collect_nullable_symbols(grammar)
     if start_symbol not in nullable_symbols:
         # the starting symbol is not nullable
         return grammar
@@ -353,9 +392,9 @@ def to_monotone_start_form(grammar: dict) -> dict:
 
 @deepcopy_arguments
 def to_chomsky_normal_form(
-    grammar: dict,
+    grammar: GrammarDict,
     order=("CHECK", "CLEAN", "START", "TERM", "BIN", "DEL", "UNIT", "CLEAN"),
-) -> dict:
+) -> GrammarDict:
     """Transform context free grammar into chomsky normal form
 
     This is done by calling other functions in a specific order.
@@ -415,7 +454,7 @@ def to_chomsky_normal_form(
 
 
 @deepcopy_arguments
-def to_clean_form(grammar: dict) -> dict:
+def to_clean_form(grammar: GrammarDict) -> GrammarDict:
     """Cleans the grammar from unusable symbols and productions
 
     This is done by first removing all productions which right hand sides can
@@ -425,8 +464,8 @@ def to_clean_form(grammar: dict) -> dict:
     Lastly all productions are scanned for used symbols and all symbols not in
     any production are removed from the grammar
 
-    :param grammar: A context-free Grammar
-    :return: the cleaned Grammar
+    :param grammar: A context-free grammar
+    :return: the cleaned grammar
     """
     return to_unused_symbol_free_form(
         to_unproductive_production_free_form(
@@ -436,7 +475,7 @@ def to_clean_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_unproductive_production_free_form(grammar: dict) -> dict:
+def to_unproductive_production_free_form(grammar: GrammarDict) -> GrammarDict:
     """This function removes all productions which ultimately cant create a word
 
     This works by searching the production set for new productions which can
@@ -448,7 +487,7 @@ def to_unproductive_production_free_form(grammar: dict) -> dict:
     nonterminals
 
     :param grammar:
-    :return:
+    :return: grammar without unnecessary productions
     """
     new_terminable = {symbol for symbol in grammar["terminals"]}
     symbol_to_producing_productions_map = group_productions_by_producible_symbols(
@@ -488,7 +527,7 @@ def to_unproductive_production_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_unreachable_production_free_form(grammar: dict) -> dict:
+def to_unreachable_production_free_form(grammar: GrammarDict) -> GrammarDict:
     """This function removes all productions which cant be reached from start
 
     This is done by saving a set of all nonterminals reachable from start.
@@ -540,7 +579,7 @@ def to_unreachable_production_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_unused_symbol_free_form(grammar: dict) -> dict:
+def to_unused_symbol_free_form(grammar: GrammarDict) -> GrammarDict:
     """This function removes all symbols which are not used by productions
 
     This simply iterates over all productions and looks for used symbols and
@@ -567,11 +606,11 @@ def to_unused_symbol_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_left_recursion_free_form(grammar: dict) -> dict:
+def to_left_recursion_free_form(grammar: GrammarDict) -> GrammarDict:
     """Remove all left recursions from context free grammar
 
     :param grammar:
-    :return:
+    :return: a new equivalent grammar without left recursions
     """
     sorted_nonterminals = sorted(grammar["nonterminals"])
     productions = grammar["productions"]
@@ -631,11 +670,11 @@ def to_left_recursion_free_form(grammar: dict) -> dict:
 
 
 @deepcopy_arguments
-def to_greibach_normal_form(grammar: dict) -> dict:
-    """Transform grammar into the greibach normal form
+def to_greibach_normal_form(grammar: GrammarDict) -> GrammarDict:
+    """Transform grammar into the Greibach normal form
 
     :param grammar:
-    :return:
+    :return: a new equivalent grammar of Greibach normal form
     """
     if not has_chomsky_normal_form(grammar):
         grammar = to_chomsky_normal_form(grammar)
